//-----------------------------------------------------------------------------
// File: platform_win8.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "platform_win8.h"

MP_Platform_WIN8::MP_Platform_WIN8()
{
}

MP_Platform_WIN8::~MP_Platform_WIN8()
{
}

// eng: Returns time in milliseconds
// rus: Возвращает время в милисекундах
unsigned long MP_Platform_WIN8::GetTick()
{
	return (unsigned long)GetTickCount64();
}

// eng: Returns path to folder with emitters
// rus: Возвращает путь к папке с ptc-файлами
const char* MP_Platform_WIN8::GetPathToPTC()
{
	if (ptc_path.empty())
	{
		ptc_path=wchar_to_utf8(Windows::ApplicationModel::Package::Current->InstalledLocation->Path->Data());

		#if (WINAPI_FAMILY==WINAPI_FAMILY_PHONE_APP)
		//ptc_path+="\\Assets\\magic_particles\\";
		ptc_path+="\\";
		#else
		ptc_path+="\\magic_particles\\";
		#endif
	}

	return ptc_path.c_str();
}

// eng: Returns path to folder with textures
// rus: Возвращает путь к папке с текстурами
const char* MP_Platform_WIN8::GetPathToTexture()
{
	if (texture_path.empty())
	{
		texture_path=GetPathToPTC();
		#if (WINAPI_FAMILY!=WINAPI_FAMILY_PHONE_APP)
		texture_path+="textures\\";
		#endif
	}

	return texture_path.c_str();
}

// eng: Returns path to folder which could be used by wrapper to store temporary files. If NULL is returned the temporary files are saved in RAM.
// rus: Возвращает путь к временной папке. Если вернуть NULL, то временные файлы будут сохраняться в ОЗУ.
const char* MP_Platform_WIN8::GetPathToTemp()
{
	return NULL;
}

// eng: Finds the first ptc-file in emitters folder
// rus: Поиск первого ptc-файла в папке с эмиттерами
const char* MP_Platform_WIN8::GetFirstFile()
{
	std::string ptc_path = GetPathToPTC();

	ptc_path+="*.ptc";
	
	const wchar_t* mask = utf8_to_wchar(ptc_path.c_str());
	hFindFile = FindFirstFileExW(mask, FindExInfoStandard, &fd, FindExSearchNameMatch, NULL, 0);
	if (hFindFile!=INVALID_HANDLE_VALUE)
	{
		file = wchar_to_utf8(fd.cFileName);
		return file.c_str();
	}
	FindClose(hFindFile);

	return NULL;
}

/*
// eng: Finds the first ptc-file in emitters folder
// rus: Поиск первого ptc-файла в папке с эмиттерами
const char* MP_Platform_WIN8::GetFirstFile()
{
	std::string ptc_path = GetPathToPTC();

	//ptc_path+="*.ptc";
	ptc_path += "\\*.*";


	const wchar_t* mask = utf8_to_wchar(ptc_path.c_str());
	hFindFile = FindFirstFileExW(mask, FindExInfoStandard, &fd, FindExSearchNameMatch, NULL, 0);
	//if (hFindFile!=INVALID_HANDLE_VALUE)
	while (hFindFile != INVALID_HANDLE_VALUE)
	{
		file = wchar_to_utf8(fd.cFileName);
		//return file.c_str();
		FindNextFileW(hFindFile, &fd);
	}
	FindClose(hFindFile);

	return NULL;
}
*/


// eng: Finds the next ptc-file in emitters folder
// rus: Поиск следующего ptc-файла в папке с эмиттерами
const char* MP_Platform_WIN8::GetNextFile()
{
	if (FindNextFileW(hFindFile,&fd))
	{
		file=wchar_to_utf8(fd.cFileName);
		return file.c_str();
	}
	FindClose(hFindFile);

	return NULL;
}

// ru: Удаление файла (аналог "remove")
// en: Deleting of file (analogue "remove")
int MP_Platform_WIN8::RemoveFile(const char* file)
{
	return _wremove(utf8_to_wchar(file));
}